@@ -29,7 +29,7 @@ sellipse <- function(a, b, k) {
 #'
 #' @param k superellipse exponent.
 #' @param res the desired resolution
-#' @return a data frame containing 
+#' @return a data frame containing x and y coordinates for the outline of the (super)ellipse
 #' @export sellipse.coo
 #' @examples
 #' sellipse.coo(2.0)->df #get coordinates for normal ellipse (exponent k=2)
@@ -145,9 +145,9 @@ ecomp <- sellipse(vdiam/2, hdiam/2, k)
 
     ## Return the calculated area or ratio
   if(return=="area"){return(area)
-  }else if(return=="area_corr"){
+  }else if(return%in%c("area_corr","corr")){
   return(area/ecomp)
-  }else if(return=="aspect_ratio"){
+  }else if(return%in%c("asp","aspect_ratio")){
   return(hdiam/vdiam)
   }else if(return=="diameters"){
   diam<-c(x=hdiam, y=vdiam)
@@ -224,9 +224,6 @@ I_corr<-c(I_x_total,I_y_total,I_z_total)/c(ellipse_x, ellipse_y, ellipse_polar)
 }
 
 
-
-
-
 ##Function measuresil()
 #'Take pixel-by-pixel measurements of a silhouette in jpeg or png format for use with the gdi function.
 #'
@@ -325,6 +322,88 @@ return(depthscenters)
 }
 
 
+
+## function inter_corr()
+#' Helper function for interpolating cross-sectional metrics over the length of a silhouette
+#' @param ... cross-sections between which to interpolate. Each object can be either a single numeric each giving the quantity to be interpolated between, a full output of cscorr() with option return="all", or a character vector giving filenames to use with cscorr().
+#' @param x indices along silhouette to use as points of interpolation. If NULL, the function will treat indices as indices of the silhouette itself, not x, and attempt to prepend 1 and append sil (or its length) to it, to make the cross-section interpolate toward the default value at the first and last slice.
+#' @param sil dataframe or vector (output of measuresil()) or single numeric giving the total number of slices of the silhouette for which to interpolate cross-sections
+#' @param indices subset of x to which the elements in ... pertain, if ... does not have the same total number of elements as x
+#' @param default default value to fill for elements of x not in indices, if length(x)>length(...)
+#' @param default_k default superellipse exponent to use for elements of x not in indices, supersedes setting for default if not NULL and return parameter is set to "area_corr"
+#' @param return which element of the cscorr() output to return in interpolated form (defaults to area_corr, can also be "asp"
+#' @param v verbosity setting (logical)
+#' @details This is a convenience function and wrapper around approx() and cscorr(), which creates a linear interpolation of cross-section geometry over the entire length of a silhouette. Input can in principle be any numeric value (but, is intended to primarily be cross-sectional geometry defined by "area_corr" and "asp" elements returned via cscorr().
+#' @importFrom stats approx
+#' @export inter_corr
+#' @examples
+#' fdir <- system.file(package="gdi")
+#' cf <- cscorr(file.path(fdir,"exdata","cross_section.png"))
+#' lat <- measuresil(file.path(fdir,"exdata","lat.png"))
+#' inter_corr(cf,x=c(1,1000,2341), sil=lat, indices=2)
+
+inter_corr<-function(...,x=NULL,sil,indices=NULL,default=1, default_k=2, return="area_corr",v=FALSE){
+list(...)->dots
+if(is.data.frame(sil)) sil<-nrow(sil)
+if(is.numeric(sil) && length(sil)>1) sil<-length(sil)
+if(is.null(x) && !is.null(indices)) c(1,indices,sil)->x
+if(v) print(sil)
+if(v) print(dots)
+
+if(length(x)!=length(dots) && is.null(indices)) stop("x is not same length as supplied cross-sections and no i was provided to subset it")
+#extract corrs
+corrs<-numeric()
+
+corr_temp<-list()
+for(i in 1:length(dots)){
+
+if(is.numeric(dots[[i]]) && length(dots[[i]])==1){
+as.numeric(dots[[i]])[1]->corrs[i]
+
+}else if(is.character(dots[[i]])){
+if(dots[[i]]%in%names(corr_temp)){
+corr_temp[[dots[[i]]]]->c0
+}else{
+cscorr(dots[[i]],return=return)->c0
+c0->corr_temp[[dots[[i]]]]
+}
+c0->corrs[i]
+
+}else if(is.numeric(dots[[i]]) && length(dots)>1){
+dots[[i]]->c0
+c0[return]->corrs[i]
+}
+
+}
+if(v) print(corrs)
+##interpolate corr factors
+if(length(x)!=length(corrs) && !is.null(indices) && length(indices)==length(corrs)){
+if(is.null(default_k) | return!="area_corr"){default->corr_default
+}else{sellipse(1,1,default_k)/sellipse(1,1,2)->corr_default}
+
+
+if(v) print(corr_default)
+y_in<-rep(corr_default,length(x))
+y_in[indices]<-corrs
+
+approx(x=x, y=y_in, xout=c(1:sil))$y->out
+
+}else{
+corrs->y_in
+approx(x=x, y=y_in, xout=c(1:sil))$y->out
+}
+
+#set information attributes
+attr(out,"x")<-x
+attr(out,"y_in")<-y_in
+
+attr(out,"return")<-return
+
+#output
+return(out)
+}##
+
+
 ##Function gdi()
 #'Estimate volume using Graphic Double Integration.
 #'
@@ -332,12 +411,12 @@ return(depthscenters)
 #' @param dors Measurements of diameter in dorsal view/second of two orthogonal views to be used with the gdi. Can be either a numeric vector, a data.frame (output of measuresil(...,return="all") with a collumn named "diameter", or a text file with diameter measurements to be scanned. Must be the same length as lat.
 #' @param indices Optional indices specifying a subset of the silhouette measurement vectors to be analyzed. Useful if separate segment calculations are desired.
 #' @param scale Scale of the data, given in terms of how many units of the input data (e.g. pixels) are in one side of the desired unit of output volume. Defaults to 10.
-#' @param sliceL Length of individual segments to be used in the GDI. Defaults to 1/scale.
+#' @param sliceL Length of individual segments to be used in the GDI. Defaults to 1.
 #' @param method Method to be used for the GDI. Default "raw" setting calculates each segment as an elliptical cylinder with volume = Area * SliceL. Any other string will result in volume being calculated as an elliptical frustum with base areas based on the measurements of segments i and i+1.
 #' @param k Superellipse exponent to be used for the cross-sectional area. Defaults to 2.0 (normal ellipse).
 #' @param corr Correction factor for area of cross-sections, calculated as the ratio between the actual cross-sectional area and that of a (super)ellipse (depending on the specified exponent k) with the same diameters. This setting enables the function to account for complex, non-elliptical cross-sections. Default value is 1, i.e. no correction. Can be either a single number, or a numeric vector of the same length as lat and dors (in the case of a changing cross-sectional geometry along the length of the body).
 #' @param smooth.ends If method != "raw", specify whether first and last segments should be left raw, or taper to 0 (i.e. be approximated as cones). Only applies if there are no leading or following zeros in the measurement vectors.
-#' @param return Determines whether to report the estimated total volume (if default/"total"), or a data.frame() with segment radii, areas and volumes (if left empty of any other character string.
+#' @param return Determines whether to report the estimated total volume (if default/"total"), or a data.frame() with segment radii, areas and volumes (if left empty of any other character string).
 #' @return Either a single number representing the total volume estimated (with names indicating the horizontal length of the silhouette in the unit determined by scale), or (if return!="total") a data.frame() containing columns with the radii in both dimensions, the estimated elliptical or superelliptical areas, and the segment volumes.
 #' @export gdi
 #' @examples
@@ -347,7 +426,7 @@ return(depthscenters)
 #' gdi(lat=lateral, dors=lateral/2, scale=10, method="smooth", k=2.3)
 
 
-gdi<-function(lat, dors, indices=NULL, scale=10, sliceL=1/scale, method="raw", k=2.0, corr=1, smooth.ends=FALSE, return="total"){
+gdi<-function(lat, dors, indices=NULL, scale=10, sliceL=1, method="raw", k=2.0, corr=1, smooth.ends=FALSE, return="total"){
 lat_<-lat
 dors_<-dors
 
@@ -367,15 +446,16 @@ if(is.data.frame(dors)){
     dors_<-dors$diameter
 }
 #make data.frame
-sil<-data.frame(ydiam_raw=lat_,zdiam_raw=dors_,ydiam_scaled=lat_/scale,zdiam_scaled=dors_/scale, slice_length=sliceL)#scale-adjust and save in dataframe
+sil<-data.frame(ydiam_raw=lat_,zdiam_raw=dors_,ydiam_scaled=lat_/scale,zdiam_scaled=dors_/scale, slice_length=sliceL/scale, sliceL_raw=sliceL)#scale-adjust and save in dataframe
+sliceL/scale->sliceL
 
 if(return!="total"){
-#save segment centroid positions
+#save raw segment centroid positions on the x (anteroposterior) axis
 ##x
-sil$x_center<-sil$slice_length*scale/2
-l<-cumsum(sil$x_center*2)
+sil$x_center<-sil$sliceL_raw/2#half the raw sliceL
+l<-cumsum(sil$x_center*2)#cumulative length of all slices
 if(length(sil$x_center)>1){
-for(i in 2:length(sil$x_center)){
+for(i in 2:length(sil$x_center)){#for each segment, subtract half the segment length from the cumulative segment length to get the segment center
 sil$x_center[i]<-l[i]-sil$x_center[i]
 }}
 
@@ -418,7 +498,7 @@ attr(sil, "z_resolution")<-attr(dors, "y_resolution")
 
 
 sum(sil$V)->res#sum up segments
-names(res) <- paste("x_dim",sum(sil$ydiam_raw!=0)/scale, "units", sep="_")
+names(res) <- paste("x_dim",sum(sil$slice_length[sil$ydiam_raw!=0]), "units", sep="_")
 
 if(return=="total"){
 return(res)
@@ -652,6 +732,62 @@ return(weighted.mean(y_center, w=masses, na.rm=TRUE)/scale)
 }
 
 
+##Function expandConvexHull()
+#' Estimate soft tissue expansion factors following Macaulay et al. 2023
+#'
+#' @param volume Numeric value containing volume of the convex hull (in liters). Defaults to NULL, in which case an isometric expansion factor is always returned. Setting for this parameter is ignored if method=="isometry"
+#' @param method Method to use for deriving soft tissue expansion factor. Possible optios are "isometry" (default), "OLS" or "PGLS".
+#' @param taxon Taxon whose specific regressions or expansion factors to use, can be "Croc_Lizard" for the expansions based on Crocodilians and Squamates, "Bird" for expansions based on birds of "All" for the pooled (allometric) or averaged (isometric) expansions.
+#' @param segment Body segment for which expansion factor should be calculated. Possible options are: "Head", "Neck", "Trunk", "Tail", "Humerus", "Forearm", "Hand", "Thigh", "Shank", "MT" or "Pes"
+#' @return A single numeric, containing the calculated expansion factor
+#' @export expandConvexHull
+#' @details This function returns a soft tissue expansion factor, calculated as the quotient of the skin volume and minimum convex hull volume of the selected body segment. This is based on expansion factors and regressions for birds and non-avian reptiles from Macaulay et al. 2023 (<10.1038/s41467-023-37317-y>). OLS and PGLS estimate the body volume allometrically following supplements 3, 4 and 5 of Macaulay et al., while isometry returns an isometric expansion factor following supplement 6. The body segment "Tail" should not be used in conjunction with the taxon "Bird", due to the absence of tails in this group – if this occurs nonethemess, an expansion factor of 1 is returned.
+#' @examples
+#' expandConvexHull(50,"isometry",taxon="Croc_Lizard",segment="Trunk")
+
+expandConvexHull<-function(volume=NULL,method="isometry",taxon="Croc_Lizard",segment="Trunk"){
+
+if(method%in%c("isometry","iso","Isometry","Iso") | is.null(volume)){
+##isometric expansions
+iso<-data.frame(Croc_Lizard=c(1.266, 8.646, 1.219, 3.369, 2.852, 2.866, 3.564, 5.102, 2.655, 3.494, 2.553),Bird=c(1.008, 3.825, 1.436, 1, 1.97, 1.736, 1.303, 4.538, 1.729, 0.792, 1.716))
+iso$all<-c(1.137,6.235,1.328,3.369,2.411,2.301,2.434,4.820,2.192,2.143,2.135)
+rownames(iso)<-c('Head', 'Neck', 'Trunk', 'Tail', 'Humerus', 'Forearm', 'Hand', 'Thigh', 'Shank', 'MT', 'Pes')
+
+F_expansion<-iso[segment,taxon]
+if(!(method%in%c("isometry","iso"))) warning("Selected method was NOT isometry, but no volume was provided; returning isometric expansion factor.")
+}
+
+if(method=="OLS"){
+##allometric expansions OLS
+if(taxon%in%c("Croc_Lizard","croc_lizard","Reptile","reptile")) ols_expansions<-data.frame(intercept=c(0.002, 0.73, 0.2, 0.387, 0.888, 0.677, 1.127, 0.951, 0.788, 0.908, 0.745),slope=c(0.978, 0.961, 1.039, 0.984, 1.09, 1.045, 1.106, 1.051, 1.075, 1.074, 1.067))
+if(taxon%in%c("bird","Bird","Aves","avian","Avian")) ols_expansions<-data.frame(intercept=c(-0.085, 0.008, 0.213,0, 0.001, 0.017, 0.124, 0.487, 0.322, -0.402, 0.21),slope=c(0.982, 0.892, 1.018,1, 0.95, 0.963, 1.012, 0.975, 1.021, 0.946, 1.002))
+if(taxon%in%c("all","All")) ols_expansions<-data.frame(intercept=c(-0.085, 0.008, 0.213, 0.001, 0.017, 0.124, 0.487, 0.322, -0.402, 0.21),slope=c(0.982, 0.892, 1.018,1, 0.95, 0.963, 1.012, 0.975, 1.021, 0.946, 1.002))
+rownames(ols_expansions)<-c('Head', 'Neck', 'Trunk', 'Tail', 'Humerus', 'Forearm', 'Hand', 'Thigh', 'Shank', 'MT', 'Pes')
+
+expansion_function<-function(x){exp(ols_expansions[segment,"slope"]*log(volume)+ols_expansions[segment,"intercept"])}
+
+F_expansion<-expansion_function(volume)/volume
+}
+
+if(method=="PGLS"){
+##allometric expansions PGLS
+if(taxon%in%c("Croc_Lizard","croc_lizard","Reptile","reptile")) pgls_expansions<-data.frame(intercept=c(0.32, 0.711, 0.3, 0.27, 0.673, 0.795, 0.935, 1.013, 0.934, 0.941, 1.083),slope=c(1.052, 0.956, 1.076, 0.959, 1.047, 1.065, 1.072, 1.063, 1.107, 1.08, 1.134))
+if(taxon%in%c("bird","Bird","Aves","avian","Avian")) pgls_expansions<-data.frame(intercept=c(-0.084, -0.048, 0.178,0, 0.03, 0.08, 0.128, 0.524, 0.33, -0.399, 0.228),slope=c(0.981, 0.879, 1.004,1, 0.951, 0.97, 1.01, 0.981, 1.023, 0.947, 1.003))
+if(taxon%in%c("all","All")) pgls_expansions<-data.frame(intercept=c(-0.054, -0.063, 0.173,0, 0.006, 0.072, 0.115, 0.507, 0.324, -0.4, 0.219),slope=c(0.981, 0.876, 1.005,1, 0.95, 0.967, 1.011, 0.978, 1.021, 0.946, 1.009))
+
+rownames(pgls_expansions)<-c('Head', 'Neck', 'Trunk', 'Tail', 'Humerus', 'Forearm', 'Hand', 'Thigh', 'Shank', 'MT', 'Pes')
+
+expansion_function<-function(x){exp(pgls_expansions[segment,"slope"]*log(volume)+pgls_expansions[segment,"intercept"])}
+
+F_expansion<-expansion_function(volume)/volume
+}
+
+return(F_expansion)
+
+#Source: SuppData 4, 5 and 6 of Macaulay, S., Hoehfurtner, T., Cross, S.R.R., Marek, R.D., Hutchinson, J.R., Schachner, E.R., Maher, A.E., and Bates, K.T. 2023. Decoupling body shape and mass distribution in birds and their dinosaurian ancestors. Nature Communications: 14:1575. https://doi.org/10.1038/s41467-023-37317-y.
+}
+##
+
 
 ##Function plot_sil()
 #' Plots a silhouette read by measuresil()
@@ -659,6 +795,7 @@ return(weighted.mean(y_center, w=masses, na.rm=TRUE)/scale)
 #' @param sil A data frame that is the output of measuresil(..., return="all"), containing the center and the diameter of the silhouette at each value for x.
 #' @param flip Logical indicating whether to flip axes (needed if measuresil() was performed using align="v", defaults to FALSE.
 #' @param add Logical indicating whether to add silhoutte to an existing plot (defaults to FALSE)
+#' @param expansion Numeric containing expansion factor to multiply with body diameters. Defaults to 1 (i.e. no expansion). XXX
 #' @param xoffset Optional value by which to shift the silhouette on the x axis
 #' @param yoffset Optional value by which to shift the silhouette on the y axis
 #' @param alpha Opacity value for fill of polygon (defaults to 1)
@@ -682,7 +819,9 @@ return(weighted.mean(y_center, w=masses, na.rm=TRUE)/scale)
 #' measuresil(file.path(fdir,"exdata","lat.png"), return="all")->lat_
 #' plot_sil(lat_)
 
-plot_sil<-function(sil, flip=FALSE, add=FALSE, xoffset=0, yoffset=0,alpha=1, col="grey", border="darkgrey", scale=1, xlab="", ylab="",...){
+plot_sil<-function(sil, flip=FALSE, add=FALSE, expansion=1, xoffset=0, yoffset=0,alpha=1, col="grey", border="darkgrey", scale=1, xlab="", ylab="",...){
+
+sil$diameter*expansion->sil$diameter#adjust diameters by expansion factor
 
 if(flip==FALSE){
 x<-c(1:nrow(sil),rev(1:nrow(sil)))+xoffset
@@ -837,7 +976,7 @@ return(total)
 #' @param y An optional vector of vertical (dorsoventral) segment COM positions.
 #' @param dors_diam An optional vector of transverse diameters of the silhouette, required if not contained in x.
 #' @param lat_diam An optional vector of vertical diameters of the silhouette, required if not contained in x. Needed if inertia for "roll" or "pitch" should be calculated.
-#' @param axis_coord An optional coordinate of the axis of rotation, defaults to the center of mass of the entire volume if not set.
+#' @param axis_coord An optional coordinate of the axis of rotation (in original units, i.e. pixels), defaults to the center of mass of the entire volume if not set.
 #' @param axis Axis of rotation, defaults to "yaw" (i.e. rotation around vertical axis), can also be "pitch" (rotation around transverse axis) or "roll" (rotation around horizontal axis). For yaw rotation, the body is assumed to be bilaterally symmetrical, whereas for pitch rotation, dorsoventral variation in COM of segments is taken into account.
 #' @param volumes An optional separate vector of volumes, required if x is not a data.frame containing them.
 #' @param corr An optional correction factor for the cross-sectional shape, given as the ratio between the characteristic mass moment of inertia of a plane with the given shape (e.g. determined by cscorr()) and an elliptical plane with the same diameters and assigned mass. Allows the calculation of moments of inertia for bodies with arbitrary cross-sectional shapes.
@@ -987,4 +1126,183 @@ return(results_roll)
 }
 
 
+}##
+
+
+
+
+
+##function transfer_ratio()
+#' Transfer a vector of aspect ratios onto another body profile
+#' @param lat diameters in lateral view, supply to estimate transverse diameters from dorsoventral ones
+#' @param dors diameters in dorsal (or ventral) view, supply to estimate dorsoventral diameters from transverse ones
+#' @param indices optional vector of indices to apply to subset lat or dors
+#' @param lat0 (dorsoventral) diameters in lateral view for template
+#' @param dors0 (transverse) diameters in dorsal/ventral view for template (vector is subsetted to same length as lat0 if lengths don’t match)
+#' @param indices0 optional vector of indices to subset lat0 and dors0
+#' @param smooth width of smoothing interval (as fraction of full silhouette length)
+#' @param return what to return, either "diameters" to return diameters, or "ratios", to return ratios
+#' @param ... additional arguments to pass on to paleoDiv::rmeana() for computation of running mean of diameter ratios
+#' @return Either a numeric vector of interpolated transverse or dorsoventral diameters or width/depth or depth/width ratios depending on the chosen settings, or a data.frame of same format as dors0 or lat0 (with the "center" column copied)
+#' @export transfer_ratio
+#' @examples
+#' fdir <- system.file(package="gdi")
+#' lat <- measuresil(file.path(fdir,"exdata","lat.png"), return="all")
+#' dors <- measuresil(file.path(fdir,"exdata","dors.png"), return="all")
+#' gdi(lat,dors,scale=100) #real volume
+#' dors_est <- transfer_ratio(lat=lat,lat0=lat,dors0=dors,smooth=0.005)
+#' gdi(lat,dors_est,scale=100) #volume with dorsal view interpolated
+#' lat_est <- transfer_ratio(dors=dors,lat0=lat,dors0=dors,smooth=0.005)
+#' gdi(lat_est,dors,scale=100) #volume with lateral view interpolated
+#' plot_sil(lat,asp=1) #visualize results
+#' plot_sil(dors,add=TRUE)
+#' plot_sil(lat_est,add=TRUE,col="blue",alpha=0.3)
+#' plot_sil(dors_est,add=TRUE,col="blue",alpha=0.3)
+
+
+transfer_ratio<-function(lat=NULL,dors=NULL,indices=NULL,lat0,dors0,indices0=NULL,smooth=0.005,return="diameters",...){
+
+#define advanced running mean function
+rmeana<-function(x0, y0, x1 = NULL, plusminus = 0.005,...){
+    if (is.null(x1)) {x_ <- x0}else{x_ <- x1}
+    y_ <- rep(NA, length(x_))
+    
+    for (i in 1:length(x_)) {
+        indices <- which(abs(as.numeric(x0 - x_[i])) <= plusminus)
+        if (length(indices) == 0) indices <- which.min(abs(x0 - x_[i]))
+		y_[i] <- mean(y0[indices], na.rm = TRUE)
+    }
+    
+    return(y_)} #end simplified rmeana function definition, see also package "paleoDiv"
+    
+#prepare template: equalize lengths
+if(is.data.frame(lat0)){
+if("center"%in%colnames(lat0)) lat0$center->lcenter
+lat0$diameter->lat0
 }
+if(is.data.frame(dors0)){
+if("center"%in%colnames(dors0)) dors0$center->dcenter
+dors0$diameter->dors0
+}
+dors0[1:length(lat0)]->dors0
+
+#subset template
+if(is.null(indices0)) c(1:length(lat0))->indices0
+lat0[indices0]->lat0
+dors0[indices0]->dors0
+if(exists("dcenter") && is.numeric(dcenter)) dcenter[indices0]->dcenter
+if(exists("lcenter") && is.numeric(lcenter)) lcenter[indices0]->lcenter
+
+#compute ratios
+dors0/lat0->ratios0
+c(1:length(lat0))->x0
+x0_rel<-x0/length(lat0)
+#
+
+##do interpolation
+if(!is.null(lat)){ ##lateral given, dorsal unknown
+df<-FALSE
+if(is.data.frame(lat)){
+df<-TRUE
+lat$diameter->lat
+}
+if(is.null(indices)) indices<-c(1:length(lat))
+lat[indices]->lat
+
+#
+c(1:length(lat))->x
+x_rel<-x/length(lat)
+
+interpolated_ratios<-rmeana(x0=c(0,x0_rel), y0=c(ratios0[1],ratios0), x1=x_rel, plusminus=smooth/2,...)
+
+
+if(any(is.na(interpolated_ratios))){ #remove nas resulting from interpolation, replace with closest non-na value
+which(is.na(interpolated_ratios))->narat
+which(!is.na(interpolated_ratios))->nnarat
+
+for(i in narat){which(abs(nnarat-i)==min(abs(nnarat-i)))->j
+interpolated_ratios[i]<-mean(interpolated_ratios[nnarat[j]],na.rm=TRUE)
+}
+
+}
+
+if(any(is.infinite(interpolated_ratios))){ #remove infs resulting from interpolation, replace with closest non-inf value
+which(is.infinite(interpolated_ratios))->narat
+which(!is.infinite(interpolated_ratios))->nnarat
+
+for(i in narat){which(abs(nnarat-i)==min(abs(nnarat-i)))->j
+interpolated_ratios[i]<-mean(interpolated_ratios[nnarat[j]],na.rm=TRUE)
+}
+
+}
+
+
+interpolated_ratios*lat->interpolated_diameters
+if(df && exists("dcenter")){
+mean(dcenter,na.rm=TRUE)->center
+rep(center,length(interpolated_diameters))->center
+}
+
+
+}else if(!is.null(dors)){ ##dorsal given, lateral unknown
+df<-FALSE
+if(is.data.frame(dors)){
+df<-TRUE
+dors$diameter->dors
+}
+if(is.null(indices)) indices<-c(1:length(dors))
+dors[indices]->dors
+
+#
+c(1:length(dors))->x
+x_rel<-x/length(dors)
+
+interpolated_ratios<-rmeana(x0=c(0,x0_rel), y0=c(ratios0[1],ratios0), x1=x_rel, plusminus=smooth/2,...)
+
+if(any(is.na(interpolated_ratios))){ #remove nas resulting from interpolation, replace with closest non-na value
+which(is.na(interpolated_ratios))->narat
+which(!is.na(interpolated_ratios))->nnarat
+
+for(i in narat){which(abs(nnarat-i)==min(abs(nnarat-i)))->j
+interpolated_ratios[i]<-mean(interpolated_ratios[nnarat[j]],na.rm=TRUE)
+}
+
+}
+
+if(any(is.infinite(interpolated_ratios))){ #remove infs resulting from interpolation, replace with closest non-inf value
+which(is.infinite(interpolated_ratios))->narat
+which(!is.infinite(interpolated_ratios))->nnarat
+
+for(i in narat){which(abs(nnarat-i)==min(abs(nnarat-i)))->j
+interpolated_ratios[i]<-mean(interpolated_ratios[nnarat[j]],na.rm=TRUE)
+}
+
+}
+
+if( df && exists("lcenter") && is.numeric(lcenter) ){
+rmeana(x0=x0_rel, y0=lcenter, x1=x_rel, plusminus=smooth/2,...)->center #interpolate lateral view center
+}
+
+dors/interpolated_ratios->interpolated_diameters
+interpolated_ratios<-1/interpolated_ratios
+}else{stop("lat or dors need to be supplied")}
+
+interpolated_diameters[is.na(interpolated_diameters)]<-0
+if(any(is.infinite(interpolated_diameters))) interpolated_diameters[is.infinite(interpolated_diameters)]<-0
+
+if(any(is.infinite(interpolated_ratios))) interpolated_ratios[is.infinite(interpolated_ratios)]<-NA
+
+#interpolated_ratios[is.na(interpolated_ratios)]<-0
+
+if(exists("center") && is.numeric(center)){
+interpolated_diameters<-data.frame(diameter=interpolated_diameters,center=center)
+interpolated_diameters$center[which(interpolated_diameters$diameter==0)]<-NA
+}
+if(return%in%c("Diameters","diameters","diam","D")) return(interpolated_diameters)
+if(return%in%c("ratios","Ratios","rat","R")) return(interpolated_ratios)
+
+}##
+
+
+
+
